import inspect
import datetime
import sys
import shutil

def log(message):
    caller_function = ""
    for func in reversed(inspect.stack()):
        if func.function != "log":
            caller_function = func.function

    print(f"[jsondatafaker][{caller_function}] - {message}")

def get_temp_filename(file_name=None):
    if file_name == None:
        file_name = "exported_file"
    
    now = datetime.datetime.now()
    file_name_date_ext = now.strftime("%Y-%m-%d_%H-%M-%S-%f")

    return f"{file_name}_{file_name_date_ext}"

def parse_null_percentge(null_percentage):
    if isinstance(null_percentage, str) and null_percentage[-1] == "%":
        null_percentage = float(null_percentage[0:-1])
    
    if isinstance(null_percentage, str) and null_percentage[0] == "%":
        null_percentage = float(null_percentage[1:])

    if isinstance(null_percentage, int) or isinstance(null_percentage, float):
        if null_percentage >= 0 and null_percentage <= 1:
            return float(null_percentage)
        elif null_percentage >= 0 and null_percentage<= 100:
            return float(null_percentage / 100)

    return float(0)

def get_file_extension(file_type):
    if file_type == "csv":
        return ".csv"
    elif file_type == "json":
        return ".json"
    elif file_type == "parquet":
        return ".parquet"
    elif file_type == "excel":
        return ".xlsx"
    else:
        return ".txt"

def progress_bar(iteration=1, lenght=1, suffix = "Complete"):
    prefix = "Progress:"
    length=50
    fill="█"
    print_end="\n"

    percent = ("{0:.1f}").format(100 * (iteration / float(lenght)))
    filled_length = int(length * iteration // lenght)
    bar = fill * filled_length + '-' * (length - filled_length)
    line_lenght = shutil.get_terminal_size((80, 20)).columns
    
    progress_bar_text = f"\r{prefix} |{bar}| {iteration}/{lenght} | {percent}% {suffix}"
    if len(progress_bar_text) > line_lenght:
        progress_bar_text = progress_bar_text[:line_lenght]
    else:
        progress_bar_text += " " * (line_lenght - len(progress_bar_text))

    sys.stdout.write(progress_bar_text)
    sys.stdout.flush()

    if iteration >= lenght:
        sys.stdout.write(print_end)
        sys.stdout.flush()


import argparse
from . import jsondatafaker

def main():
    parser = argparse.ArgumentParser(description=get_description())
    parser.add_argument('--config', required=True, help='Config yaml file path')
    parser.add_argument('--target', required=False, help='Target folder/file')

    args = parser.parse_args()

    config_file_path = None
    target_file_path = None

    if args.config is not None:
        config_file_path = args.config
    else:
        print("Missing --config parameter. Use --help for more detail.")
        return

    if args.target is not None:
        target_file_path = args.target
    else:
        target_file_path = "."  


    if isinstance(config_file_path, str):
        jsondatafaker.to_json(config_file_path, target_file_path)
    else:
        print("Wrong paramater(s)")
        print(get_description())

def get_description():
    return "more detail: https://github.com/necatiarslan/json-datafaker/blob/main/README.md"

if __name__ == '__main__':
    main()






import yaml
from os import path
from . import util

class Config:
    _instance = None
    file_path:str

    def __new__(cls, file_path:str):
            if cls._instance is None:
                cls._instance = super(Config, cls).__new__(cls)
                if not path.isabs(file_path):
                    file_path = path.abspath(file_path)

                util.log(f"received config is {file_path}")
                cls._instance.file_path = file_path
                cls._instance.load_config_file()
                cls._instance.validate_config()

            return cls._instance
    
    @staticmethod
    def get_current():
        if Config._instance is None:
            raise Exception(f"Config is not initialized yet")
        
        return Config._instance

    def load_config_file(self):
        if path.isfile(self.file_path):
            with open(self.file_path, "r") as file:
                self.config = yaml.safe_load(file)
        else:
            raise Exception(f"{self.file_path} file not found")
    
    def validate_config(self):
        if "json" not in self.config:
            raise Exception(f"Config file should have json node")
        
        util.log(f"config file is validated")


    def get_locale(self):
        if "config" in self.config and "locale" in self.config["config"]:
            return self.config["config"]["locale"]
        else:
            return "en_US"




import sys, os
sys.path.append(os.path.abspath("."))

from jsondatafaker import jsondatafaker
from faker_education import SchoolProvider
from faker import Faker

fake = Faker()
def get_level():
    return f"level {fake.random_int(1, 5)}"



directory_path = 'tests/exports'
[os.remove(os.path.join(directory_path, file)) for file in os.listdir(directory_path) if os.path.isfile(os.path.join(directory_path, file))]

#print(fake.school_name())

#jsondatafaker.to_json("tests/test_json.yaml", "./tests/exports", fake_provider=SchoolProvider, custom_function=get_level)

json_data = jsondatafaker.generate_json("tests/test_json.yaml", fake_provider=SchoolProvider, custom_function=get_level)
print(json_data["first_name"])


from faker import Faker
import random
import json
from os import path
import datetime
from . import config, util

def to_json(config_file_path, target_file_path=None, **kwargs) -> str :
    json_data = generate_json(config_file_path, **kwargs)

    file_type = "json"
    if path.isdir(target_file_path):
            temp_file_path = path.join(target_file_path, util.get_temp_filename() + util.get_file_extension(file_type))
            write_to_file(json_data, temp_file_path)
            util.log(f"data is exported to {temp_file_path} as {file_type}")
            result = temp_file_path 
    else:
        write_to_file(json_data, target_file_path)
        util.log(f"data is exported to {target_file_path} as {file_type}")
        result = target_file_path
    
    return result

def write_to_file(json_data, target_file_path):
    with open(target_file_path, mode="w") as file:
        json.dump(json_data, file, indent=4)

def generate_json(config_file_path, **kwargs):
    configurator = config.Config(config_file_path)
    json_config = configurator.config["json"]

    locale = configurator.get_locale()
    fake = Faker(locale)

    if "fake_provider" in kwargs:
        if not isinstance(kwargs["fake_provider"], list):
            fake.add_provider(kwargs["fake_provider"])
        else:
            for provider in kwargs["fake_provider"]:
                fake.add_provider(provider)

    def generate_data(node):
        if isinstance(node, dict):
            return {key: generate_data(value) for key, value in node.items()}
        elif isinstance(node, list):
            return [generate_data(item) for item in node]
        elif isinstance(node, str) and node == 'None':
            return None
        elif isinstance(node, str):
            return generate_fake_value(fake, node, **kwargs)
        else:
            return node

    fake_data = generate_data(json_config)
    util.log(f"json data is generated")
    
    return fake_data

def generate_fake_value(fake: Faker, command, **kwargs):
    result = None

    variables = {
        "random": random,
        "fake": fake,
        "result": result,
        "command": command
        }
    
    if "custom_function" in kwargs:
        if isinstance(kwargs["custom_function"], list):
            for func in kwargs["custom_function"]:
                variables[func.__name__] = func
        else:
            func = kwargs["custom_function"]
            variables[func.__name__] = func
    
    exec(f"result = {command}", variables)
    result = variables["result"]

    if isinstance(result, (datetime.date, datetime.datetime)):
        result = result.isoformat()

    return result



from setuptools import setup, find_packages

with open("README.md") as file:
    description = file.read()

setup(
    name='jsondatafaker',
    version='1.0.1',
    description='Generate and Save Fake JSON defining the schema with Yaml',
    long_description = description,
    long_description_content_type = "text/markdown",
    author='Necati Arslan',
    author_email='necatia@gmail.com',
    packages=find_packages(),
    install_requires=[
        'faker',
        'pyyaml'
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Topic :: Software Development :: Libraries',
    ],
    project_urls={
        "Documentation": "https://github.com/necatiarslan/json-datafaker/blob/main/README.md",
        "Source": "https://github.com/necatiarslan/json-datafaker",
    },
    entry_points={
        'console_scripts': [
            'jsondatafaker = jsondatafaker.cli:main',
        ],
    },
)